@@ -103,7 +103,7 @@ fi
 
 DATA="$WSREP_SST_OPT_DATA"
 
-INFO_FILE='xtrabackup_galera_info'
+INFO_FILE='mariadb_backup_galera_info'
 DONOR_INFO_FILE='donor_galera_info'
 IST_FILE='xtrabackup_ist'
 MAGIC_FILE="$DATA/$INFO_FILE"
@@ -714,7 +714,7 @@ cleanup_at_exit()
     fi
 
     # Final cleanup
-    pgid=$(ps -o 'pgid=' $$ 2>/dev/null | grep -o -E '[0-9]+' || :)
+    pgid=$(ps -o pgid= $$ 2>/dev/null | grep -o -E '[0-9]+' || :)
 
     # This means no setsid done in mysqld.
     # We don't want to kill mysqld here otherwise.
@@ -1086,27 +1086,27 @@ if [ "$WSREP_SST_OPT_ROLE" = 'donor' ]; then
         tmpdir=$(parse_cnf "$encgroups" 'tmpdir')
         if [ -z "$tmpdir" ]; then
             xtmpdir="$(mktemp -d)"
-            itmpdir="$(mktemp -d)"
         elif [ "$OS" = 'Linux' ]; then
-            xtmpdir=$(mktemp -d "--tmpdir=$tmpdir")
-            itmpdir=$(mktemp -d "--tmpdir=$tmpdir")
+            xtmpdir=$(mktemp '-d' "--tmpdir=$tmpdir")
         else
-            xtmpdir=$(TMPDIR="$tmpdir"; mktemp -d)
-            itmpdir=$(TMPDIR="$tmpdir"; mktemp -d)
+            xtmpdir=$(TMPDIR="$tmpdir"; mktemp '-d')
         fi
 
         wsrep_log_info "Using '$xtmpdir' as mariadb-backup temporary directory"
         tmpopts=" --tmpdir='$xtmpdir'"
 
-        wsrep_log_info "Using '$itmpdir' as mariadb-backup working directory"
+        itmpdir="$(mktemp -d)"
+        wsrep_log_info "Using '$itmpdir' as mariadb-abackup working directory"
 
+        usrst=0
         if [ -n "$WSREP_SST_OPT_USER" ]; then
            INNOEXTRA="$INNOEXTRA --user='$WSREP_SST_OPT_USER'"
+           usrst=1
         fi
 
         if [ -n "$WSREP_SST_OPT_PSWD" ]; then
             export MYSQL_PWD="$WSREP_SST_OPT_PSWD"
-        elif [ -n "$WSREP_SST_OPT_USER" ]; then
+        elif [ $usrst -eq 1 ]; then
             # Empty password, used for testing, debugging etc.
             unset MYSQL_PWD
         fi
@@ -1167,12 +1167,6 @@ if [ "$WSREP_SST_OPT_ROLE" = 'donor' ]; then
 
         iopts="--databases-exclude='lost+found'${iopts:+ }$iopts"
 
-        if [ ${FORCE_FTWRL:-0} -eq 1 ]; then
-            wsrep_log_info "Forcing FTWRL due to environment variable" \
-                           "FORCE_FTWRL equal to $FORCE_FTWRL"
-            iopts="--no-backup-locks${iopts:+ }$iopts"
-        fi
-
         # if compression is enabled for backup files, then add the
         # appropriate options to the mariadb-backup command line:
         if [ "$compress" != 'none' ]; then
@@ -1342,9 +1336,9 @@ else # joiner
 
     [ -f "$DATA/$IST_FILE" ] && rm -f "$DATA/$IST_FILE"
 
-    # May need xtrabackup_checkpoints later on
+    # May need mariadb_backup_checkpoints later on
     [ -f "$DATA/xtrabackup_binary" ]      && rm -f "$DATA/xtrabackup_binary"
-    [ -f "$DATA/xtrabackup_galera_info" ] && rm -f "$DATA/xtrabackup_galera_info"
+    [ -f "$DATA/mariadb_backup_galera_info" ] && rm -f "$DATA/mariadb_backup_galera_info"
 
     ADDR="$WSREP_SST_OPT_HOST"
 
@@ -1456,14 +1450,14 @@ else # joiner
         wsrep_log_info "Waiting for SST streaming to complete!"
         monitor_process $jpid
 
-        if [ ! -s "$DATA/xtrabackup_checkpoints" ]; then
-            wsrep_log_error "xtrabackup_checkpoints missing," \
+        if [ ! -s "$DATA/mariadb_backup_checkpoints" ]; then
+            wsrep_log_error "mariadb_backup_checkpoints missing," \
                             "failed mariadb-backup/SST on donor"
             exit 2
         fi
 
         # Compact backups are not supported by mariadb-backup
-        if grep -qw -F 'compact = 1' "$DATA/xtrabackup_checkpoints"; then
+        if grep -qw -F 'compact = 1' "$DATA/mariadb_backup_checkpoints"; then
             wsrep_log_info "Index compaction detected"
             wsrel_log_error "Compact backups are not supported by mariadb-backup"
             exit 2
@@ -1527,14 +1521,14 @@ else # joiner
         if [ -n "$WSREP_SST_OPT_BINLOG" ]; then
             cd "$DATA"
             binlogs=""
-            if [ -f 'xtrabackup_binlog_info' ]; then
+            if [ -f 'mariadb_backup_binlog_info' ]; then
                 NL=$'\n'
                 while read bin_string || [ -n "$bin_string" ]; do
                     bin_file=$(echo "$bin_string" | cut -f1)
                     if [ -f "$bin_file" ]; then
                         binlogs="$binlogs${binlogs:+$NL}$bin_file"
                     fi
-                done < 'xtrabackup_binlog_info'
+                done < 'mariadb_backup_binlog_info'
             else
                 binlogs=$(ls -d -1 "$binlog_base".[0-9]* 2>/dev/null || :)
             fi
