@@ -35,6 +35,7 @@ class sp_head;
 class sp_package;
 class sp_pcontext;
 class sp_name;
+class sp_expr_lex;
 class Database_qualified_name;
 struct st_sp_chistics;
 class Stored_program_creation_ctx;
@@ -156,6 +157,8 @@ public:
   }
   virtual enum_sp_type type() const= 0;
   virtual LEX_CSTRING type_lex_cstring() const= 0;
+  virtual enum_sql_command sqlcom_create() const= 0;
+  virtual enum_sql_command sqlcom_drop() const= 0;
   virtual LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const
   {
     static LEX_CSTRING m_empty_body= {STRING_WITH_LEN("???")};
@@ -185,7 +188,7 @@ public:
   }
   virtual bool add_instr_freturn(THD *thd, sp_head *sp,
                                  sp_pcontext *spcont,
-                                 Item *item, LEX *lex) const;
+                                 Item *item, sp_expr_lex *lex) const;
   virtual bool add_instr_preturn(THD *thd, sp_head *sp,
                                  sp_pcontext *spcont) const;
 
@@ -255,6 +258,8 @@ public:
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("PROCEDURE")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const { return SQLCOM_CREATE_PROCEDURE; }
+  enum_sql_command sqlcom_drop() const { return SQLCOM_DROP_PROCEDURE; }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const;
   const char *show_create_routine_col1_caption() const
   {
@@ -305,6 +310,8 @@ public:
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("FUNCTION")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const { return SQLCOM_CREATE_FUNCTION; }
+  enum_sql_command sqlcom_drop() const { return SQLCOM_DROP_FUNCTION; }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const;
   const char *show_create_routine_col1_caption() const
   {
@@ -324,7 +331,7 @@ public:
   HASH *get_priv_hash() const;
 #endif
   bool add_instr_freturn(THD *thd, sp_head *sp, sp_pcontext *spcont,
-                         Item *item, LEX *lex) const;
+                         Item *item, sp_expr_lex *lex) const;
 };
 
 
@@ -374,6 +381,8 @@ public:
     static LEX_CSTRING m_type_str= {STRING_WITH_LEN("PACKAGE")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const { return SQLCOM_CREATE_PACKAGE; }
+  enum_sql_command sqlcom_drop() const { return SQLCOM_DROP_PACKAGE; }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const
   {
     static LEX_CSTRING m_empty_body= {STRING_WITH_LEN("BEGIN END")};
@@ -407,6 +416,8 @@ public:
     static LEX_CSTRING m_type_str= {STRING_WITH_LEN("PACKAGE BODY")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const { return SQLCOM_CREATE_PACKAGE_BODY; }
+  enum_sql_command sqlcom_drop() const { return SQLCOM_DROP_PACKAGE_BODY; }
   LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const
   {
     static LEX_CSTRING m_empty_body= {STRING_WITH_LEN("BEGIN END")};
@@ -440,6 +451,8 @@ public:
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("TRIGGER")};
     return m_type_str;
   }
+  enum_sql_command sqlcom_create() const { return SQLCOM_CREATE_TRIGGER; }
+  enum_sql_command sqlcom_drop() const { return SQLCOM_DROP_TRIGGER; }
   MDL_key::enum_mdl_namespace get_mdl_type() const
   {
     DBUG_ASSERT(0);
@@ -581,7 +594,7 @@ enum
 
 /* Drop all routines in database 'db' */
 int
-sp_drop_db_routines(THD *thd, const char *db);
+sp_drop_db_routines(THD *thd, const LEX_CSTRING &db);
 
 /**
    Acquires exclusive metadata lock on all stored routines in the
@@ -593,7 +606,7 @@ sp_drop_db_routines(THD *thd, const char *db);
    @retval  false  Success
    @retval  true   Failure
  */
-bool lock_db_routines(THD *thd, const char *db);
+bool lock_db_routines(THD *thd, const Lex_ident_db_normalized &db);
 
 /**
   Structure that represents element in the set of stored routines
