@@ -290,9 +290,8 @@ public:
       str->append(tmp);
       str->append('\'');
     }
-    Item *do_get_copy(THD *thd) const override
+    Item *get_copy(THD *thd) override
     { return get_item_copy<Item_literal_fbt>(thd, this); }
-    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 
     // Non-overriding methods
     void set_value(const Fbt &value)
@@ -583,7 +582,7 @@ public:
       return do_field_string;
     }
 
-    static void do_field_fbt_native_to_binary(Copy_field *copy)
+    static void do_field_fbt_native_to_binary(const Copy_field *copy)
     {
       NativeBuffer<FbtImpl::binary_length()+1> res;
       copy->from_field->val_native(&res);
@@ -870,9 +869,8 @@ public:
     {
       return Item::save_in_field(field, no_conversions);
     }
-    Item *do_get_copy(THD *thd) const override
+    Item *get_copy(THD *thd) override
     { return get_item_copy<Item_copy_fbt>(thd, this); }
-    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
   };
 
   class Item_char_typecast_func_handler_fbt_to_binary:
@@ -972,7 +970,7 @@ public:
       Fbt_null tmp(args[0]);
       return null_value= tmp.is_null() || tmp.to_native(to);
     }
-    Item *do_get_copy(THD *thd) const override
+    Item *get_copy(THD *thd) override
     { return get_item_copy<Item_typecast_fbt>(thd, this); }
   };
 
@@ -982,10 +980,9 @@ public:
   public:
     Item_cache_fbt(THD *thd)
      :Item_cache(thd, singleton()) { }
-    Item *do_get_copy(THD *thd) const override
+    Item *get_copy(THD *thd)
     { return get_item_copy<Item_cache_fbt>(thd, this); }
-    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
-    bool cache_value() override
+    bool cache_value()
     {
       if (!example)
         return false;
@@ -995,54 +992,54 @@ public:
                                                    &m_value, type_handler());
       return true;
     }
-    String* val_str(String *to) override
+    String* val_str(String *to)
     {
       if (!has_value())
         return NULL;
       Fbt_null tmp(m_value.ptr(), m_value.length());
       return tmp.is_null() || tmp.to_string(to) ? NULL : to;
     }
-    my_decimal *val_decimal(my_decimal *to) override
+    my_decimal *val_decimal(my_decimal *to)
     {
       if (!has_value())
         return NULL;
       my_decimal_set_zero(to);
       return to;
     }
-    longlong val_int() override
+    longlong val_int()
     {
       if (!has_value())
         return 0;
       return 0;
     }
-    double val_real() override
+    double val_real()
     {
       if (!has_value())
         return 0;
       return 0;
     }
-    longlong val_datetime_packed(THD *) override
+    longlong val_datetime_packed(THD *thd)
     {
       DBUG_ASSERT(0);
       if (!has_value())
         return 0;
       return 0;
     }
-    longlong val_time_packed(THD *) override
+    longlong val_time_packed(THD *thd)
     {
       DBUG_ASSERT(0);
       if (!has_value())
         return 0;
       return 0;
     }
-    bool get_date(THD *, MYSQL_TIME *ltime, date_mode_t) override
+    bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate)
     {
       if (!has_value())
         return true;
       set_zero_time(ltime, MYSQL_TIMESTAMP_TIME);
       return false;
     }
-    bool val_native(THD *, Native *to) override
+    bool val_native(THD *thd, Native *to)
     {
       if (!has_value())
         return true;
