@@ -340,7 +340,8 @@ bool THD::open_temporary_table(TABLE_LIST *tl)
   */
   DBUG_ASSERT(!tl->derived);
   DBUG_ASSERT(!tl->schema_table);
-  DBUG_ASSERT(has_temporary_tables());
+  DBUG_ASSERT(has_temporary_tables() ||
+              (rgi_slave && rgi_slave->is_parallel_exec));
 
   if (tl->open_type == OT_BASE_ONLY)
   {
@@ -426,7 +427,7 @@ bool THD::open_temporary_table(TABLE_LIST *tl)
 #endif
 
   table->query_id= query_id;
-  thread_specific_used= true;
+  used|= THREAD_SPECIFIC_USED;
 
   /* It is neither a derived table nor non-updatable view. */
   tl->updatable= true;
@@ -626,8 +627,7 @@ bool THD::drop_temporary_table(TABLE *table, bool *is_trans, bool delete_table)
                           table->s->db.str, table->s->table_name.str));
 
   // close all handlers in case it is statement abort and some can be left
-  if (is_error())
-    table->file->ha_reset();
+  table->file->ha_reset();
 
   locked= lock_temporary_tables();
 
@@ -878,12 +878,20 @@ void THD::restore_tmp_table_share(TMP_TABLE_SHARE *share)
 bool THD::has_temporary_tables()
 {
   DBUG_ENTER("THD::has_temporary_tables");
-  bool result=
+  bool result;
 #ifdef HAVE_REPLICATION
-    rgi_slave ? (rgi_slave->rli->save_temporary_tables &&
-                 !rgi_slave->rli->save_temporary_tables->is_empty()) :
+  if (rgi_slave)
+  {
+    mysql_mutex_lock(&rgi_slave->rli->data_lock);
+    result= rgi_slave->rli->save_temporary_tables &&
+      !rgi_slave->rli->save_temporary_tables->is_empty();
+    mysql_mutex_unlock(&rgi_slave->rli->data_lock);
+  }
+  else
 #endif
-    has_thd_temporary_tables();
+  {
+    result= has_thd_temporary_tables();
+  }
   DBUG_RETURN(result);
 }
 
@@ -1077,8 +1085,13 @@ TABLE *THD::find_temporary_table(const char *key, uint key_length,
       {
         share->all_tmp_tables.remove(table);
         free_temporary_table(table);
-        it.rewind();
-        continue;
+        if (share->all_tmp_tables.is_empty())
+          table= open_temporary_table(share, share->table_name.str);
+        else
+        {
+          it.rewind();
+          continue;
+        }
       }
       result= table;
       break;
@@ -1349,7 +1362,7 @@ bool THD::log_events_and_free_tmp_shares()
   {
     if (IS_USER_TABLE(share))
     {
-      bool save_thread_specific_used= thread_specific_used;
+      used_t save_thread_specific_used= used & THREAD_SPECIFIC_USED;
       my_thread_id save_pseudo_thread_id= variables.pseudo_thread_id;
       char db_buf[FN_REFLEN];
       String db(db_buf, sizeof(db_buf), system_charset_info);
@@ -1399,7 +1412,7 @@ bool THD::log_events_and_free_tmp_shares()
         clear_error();
         CHARSET_INFO *cs_save= variables.character_set_client;
         variables.character_set_client= system_charset_info;
-        thread_specific_used= true;
+        used|= THREAD_SPECIFIC_USED;
 
         Query_log_event qinfo(this, s_query.ptr(),
             s_query.length() - 1 /* to remove trailing ',' */,
@@ -1432,7 +1445,7 @@ bool THD::log_events_and_free_tmp_shares()
         get_stmt_da()->set_overwrite_status(false);
       }
       variables.pseudo_thread_id= save_pseudo_thread_id;
-      thread_specific_used= save_thread_specific_used;
+      used = (used & ~THREAD_SPECIFIC_USED) | save_thread_specific_used;
     }
     else
     {
