@@ -73,55 +73,55 @@ class federatedx_io_mysql :public federatedx_io
   bool test_all_restrict() const;
 public:
   federatedx_io_mysql(FEDERATEDX_SERVER *);
-  ~federatedx_io_mysql() override;
+  ~federatedx_io_mysql();
 
   int simple_query(const char *fmt, ...);
-  int query(const char *buffer, size_t length) override;
-  FEDERATEDX_IO_RESULT *store_result() override;
+  int query(const char *buffer, size_t length);
+  virtual FEDERATEDX_IO_RESULT *store_result();
 
-  size_t max_query_size() const override;
+  virtual size_t max_query_size() const;
 
-  my_ulonglong affected_rows() const override;
-  my_ulonglong last_insert_id() const override;
+  virtual my_ulonglong affected_rows() const;
+  virtual my_ulonglong last_insert_id() const;
 
-  int error_code() override;
-  const char *error_str() override;
+  virtual int error_code();
+  virtual const char *error_str();
 
-  void reset() override;
-  int commit() override;
-  int rollback() override;
+  void reset();
+  int commit();
+  int rollback();
 
-  int savepoint_set(ulong sp) override;
-  ulong savepoint_release(ulong sp) override;
-  ulong savepoint_rollback(ulong sp) override;
-  void savepoint_restrict(ulong sp) override;
+  int savepoint_set(ulong sp);
+  ulong savepoint_release(ulong sp);
+  ulong savepoint_rollback(ulong sp);
+  void savepoint_restrict(ulong sp);
 
-  ulong last_savepoint() const override;
-  ulong actual_savepoint() const override;
-  bool is_autocommit() const override;
+  ulong last_savepoint() const;
+  ulong actual_savepoint() const;
+  bool is_autocommit() const;
 
   bool table_metadata(ha_statistics *stats, const char *table_name,
-                      uint table_name_length, uint flag) override;
+                      uint table_name_length, uint flag);
 
   /* resultset operations */
 
-  void free_result(FEDERATEDX_IO_RESULT *io_result) override;
-  unsigned int get_num_fields(FEDERATEDX_IO_RESULT *io_result) override;
-  my_ulonglong get_num_rows(FEDERATEDX_IO_RESULT *io_result) override;
-  FEDERATEDX_IO_ROW *fetch_row(FEDERATEDX_IO_RESULT *io_result,
-                                       FEDERATEDX_IO_ROWS **current= NULL) override;
-  ulong *fetch_lengths(FEDERATEDX_IO_RESULT *io_result) override;
-  const char *get_column_data(FEDERATEDX_IO_ROW *row,
-                                      unsigned int column) override;
-  bool is_column_null(const FEDERATEDX_IO_ROW *row,
-                              unsigned int column) const override;
+  virtual void free_result(FEDERATEDX_IO_RESULT *io_result);
+  virtual unsigned int get_num_fields(FEDERATEDX_IO_RESULT *io_result);
+  virtual my_ulonglong get_num_rows(FEDERATEDX_IO_RESULT *io_result);
+  virtual FEDERATEDX_IO_ROW *fetch_row(FEDERATEDX_IO_RESULT *io_result,
+                                       FEDERATEDX_IO_ROWS **current= NULL);
+  virtual ulong *fetch_lengths(FEDERATEDX_IO_RESULT *io_result);
+  virtual const char *get_column_data(FEDERATEDX_IO_ROW *row,
+                                      unsigned int column);
+  virtual bool is_column_null(const FEDERATEDX_IO_ROW *row,
+                              unsigned int column) const;
 
-  size_t get_ref_length() const override;
-  void mark_position(FEDERATEDX_IO_RESULT *io_result,
-                             void *ref, FEDERATEDX_IO_ROWS *current) override;
-  int seek_position(FEDERATEDX_IO_RESULT **io_result,
-                            const void *ref) override;
-  void set_thd(void *thd) override;
+  virtual size_t get_ref_length() const;
+  virtual void mark_position(FEDERATEDX_IO_RESULT *io_result,
+                             void *ref, FEDERATEDX_IO_ROWS *current);
+  virtual int seek_position(FEDERATEDX_IO_RESULT **io_result,
+                            const void *ref);
+  virtual void set_thd(void *thd);
 };
 
 
@@ -212,7 +212,7 @@ ulong federatedx_io_mysql::last_savepoint() const
 ulong federatedx_io_mysql::actual_savepoint() const
 {
   SAVEPT *savept= NULL;
-  uint index= savepoints.elements;
+  size_t index= savepoints.elements;
   DBUG_ENTER("federatedx_io_mysql::last_savepoint");
 
   while (index)
@@ -286,7 +286,7 @@ ulong federatedx_io_mysql::savepoint_release(ulong sp)
 ulong federatedx_io_mysql::savepoint_rollback(ulong sp)
 {
   SAVEPT *savept;
-  uint index;
+  size_t index;
   DBUG_ENTER("federatedx_io_mysql::savepoint_release");
   DBUG_PRINT("info",("savepoint=%lu", sp));
   
@@ -321,7 +321,7 @@ ulong federatedx_io_mysql::savepoint_rollback(ulong sp)
 void federatedx_io_mysql::savepoint_restrict(ulong sp)
 {
   SAVEPT *savept;
-  uint index= savepoints.elements;
+  size_t index= savepoints.elements;
   DBUG_ENTER("federatedx_io_mysql::savepoint_restrict");
   
   while (index)
@@ -361,7 +361,7 @@ bool federatedx_io_mysql::test_all_restrict() const
 {
   bool result= FALSE;
   SAVEPT *savept;
-  uint index= savepoints.elements;
+  size_t index= savepoints.elements;
   DBUG_ENTER("federatedx_io_mysql::test_all_restrict");
   
   while (index)
@@ -429,6 +429,7 @@ int federatedx_io_mysql::actual_query(const char *buffer, size_t length)
   if (!mysql.net.vio)
   {
     my_bool my_true= 1;
+    my_bool my_false= 0;
 
     if (!(mysql_init(&mysql)))
       DBUG_RETURN(-1);
@@ -440,15 +441,11 @@ int federatedx_io_mysql::actual_query(const char *buffer, size_t length)
     */
     /* this sets the csname like 'set names utf8' */
     mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, get_charsetname());
-    mysql_options(&mysql, MYSQL_OPT_USE_THREAD_SPECIFIC_MEMORY,
-                  (char*) &my_true);
+    mysql_options(&mysql, MYSQL_OPT_USE_THREAD_SPECIFIC_MEMORY, &my_true);
+    mysql_options(&mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &my_false);
 
-    if (!mysql_real_connect(&mysql,
-                            get_hostname(),
-                            get_username(),
-                            get_password(),
-                            get_database(),
-                            get_port(),
+    if (!mysql_real_connect(&mysql, get_hostname(), get_username(),
+                            get_password(), get_database(), get_port(),
                             get_socket(), 0))
       DBUG_RETURN(ER_CONNECT_TO_FOREIGN_DATA_SOURCE);
 
