@@ -381,13 +381,14 @@ we put it to free list to be used.
   * scan whole LRU list
   * scan LRU list even if buf_pool.try_LRU_scan is not set
 
-@param have_mutex  whether buf_pool.mutex is already being held
-@return the free control block, in state BUF_BLOCK_MEMORY */
-buf_block_t *buf_LRU_get_free_block(bool have_mutex)
+@param get  how to allocate the block
+@return the free control block, in state BUF_BLOCK_MEMORY
+@retval nullptr if get==have_no_mutex_soft and memory was not available */
+buf_block_t* buf_LRU_get_free_block(buf_LRU_get get)
 {
   bool waited= false;
   MONITOR_INC(MONITOR_LRU_GET_FREE_SEARCH);
-  if (!have_mutex)
+  if (UNIV_LIKELY(get != have_mutex))
     mysql_mutex_lock(&buf_pool.mutex);
 
   buf_LRU_check_size_of_non_data_objects();
@@ -406,32 +407,17 @@ got_block:
     ut_ad(LRU_size <= BUF_LRU_MIN_LEN ||
           available >= scan_depth || buf_pool.need_LRU_eviction());
 
-    ut_d(bool signalled = false);
-
     if (UNIV_UNLIKELY(available < scan_depth) && LRU_size > BUF_LRU_MIN_LEN)
     {
       mysql_mutex_lock(&buf_pool.flush_list_mutex);
       if (!buf_pool.page_cleaner_active())
-      {
         buf_pool.page_cleaner_wakeup(true);
-        ut_d(signalled = true);
-      }
       mysql_mutex_unlock(&buf_pool.flush_list_mutex);
     }
 
-    if (!have_mutex)
+    if (UNIV_LIKELY(get != have_mutex))
       mysql_mutex_unlock(&buf_pool.mutex);
 
-    DBUG_EXECUTE_IF("ib_free_page_sleep",
-    {
-      static bool do_sleep = true;
-      if (do_sleep && signalled)
-      {
-        std::this_thread::sleep_for(std::chrono::milliseconds(10));
-        do_sleep = false;
-      }
-    });
-
     block->page.zip.clear();
     return block;
   }
@@ -451,6 +437,12 @@ got_block:
     buf_pool.try_LRU_scan= false;
   }
 
+  if (get == have_no_mutex_soft)
+  {
+    mysql_mutex_unlock(&buf_pool.mutex);
+    return nullptr;
+  }
+
   waited= true;
 
   while (!(block= buf_LRU_get_free_only()))
@@ -466,10 +458,7 @@ got_block:
     mysql_mutex_unlock(&buf_pool.flush_list_mutex);
     if (my_cond_timedwait(&buf_pool.done_free, &buf_pool.mutex.m_mutex,
                           &abstime))
-    {
       buf_pool.LRU_warn();
-      buf_LRU_check_size_of_non_data_objects();
-    }
   }
 
   goto got_block;
@@ -783,7 +772,7 @@ bool buf_LRU_free_page(buf_page_t *bpage, bool zip)
 
 	/* We must hold an exclusive hash_lock to prevent
 	bpage->can_relocate() from changing due to a concurrent
-	execution of buf_page_get_low(). */
+	execution of buf_page_get_gen(). */
 	buf_pool_t::hash_chain& chain= buf_pool.page_hash.cell_get(id.fold());
 	page_hash_latch& hash_lock = buf_pool.page_hash.lock_get(chain);
 	/* We cannot use transactional_lock_guard here,
@@ -828,7 +817,6 @@ relocate_compressed:
 			ut_d(uint32_t s=) b->fix();
 			ut_ad(s == buf_page_t::FREED
 			      || s == buf_page_t::UNFIXED
-			      || s == buf_page_t::IBUF_EXIST
 			      || s == buf_page_t::REINIT);
 		}
 		break;
@@ -1299,6 +1287,51 @@ bool buf_LRU_scan_and_free_block(ulint limit)
     buf_LRU_free_from_common_LRU_list(limit);
 }
 
+void buf_LRU_truncate_temp(uint32_t threshold)
+{
+  /* Set the extent descriptor page state as FREED */
+  for (uint32_t cur_xdes_page= xdes_calc_descriptor_page(
+         0, fil_system.temp_space->free_limit);
+       cur_xdes_page >= threshold;)
+  {
+    mtr_t mtr;
+    mtr.start();
+    if (buf_block_t* block= buf_page_get_gen(
+          page_id_t(SRV_TMP_SPACE_ID, cur_xdes_page), 0, RW_X_LATCH,
+          nullptr, BUF_PEEK_IF_IN_POOL, &mtr))
+    {
+      uint32_t state= block->page.state();
+      ut_ad(state > buf_page_t::UNFIXED);
+      ut_ad(state < buf_page_t::READ_FIX);
+      block->page.set_freed(state);
+    }
+    cur_xdes_page-= uint32_t(srv_page_size);
+    mtr.commit();
+  }
+
+  const page_id_t limit{SRV_TMP_SPACE_ID, threshold};
+  mysql_mutex_lock(&buf_pool.mutex);
+  for (buf_page_t* bpage = UT_LIST_GET_FIRST(buf_pool.LRU);
+       bpage;)
+  {
+    buf_page_t* next= UT_LIST_GET_NEXT(LRU, bpage);
+    if (bpage->id() >= limit)
+    {
+    #ifdef UNIV_DEBUG
+      if (bpage->lock.u_lock_try(0))
+      {
+        ut_ad(bpage->state() == buf_page_t::FREED);
+        bpage->lock.u_unlock();
+      }
+    #endif /* UNIV_DEBUG */
+      ut_ad(!reinterpret_cast<buf_block_t*>(bpage)->index);
+      buf_LRU_free_page(bpage, true);
+    }
+    bpage= next;
+  }
+  mysql_mutex_unlock(&buf_pool.mutex);
+}
+
 #ifdef UNIV_DEBUG
 /** Validate the LRU list. */
 void buf_LRU_validate()
