@@ -342,7 +342,7 @@ static bool fil_node_open_file_low(fil_node_t *node)
   ut_ad(node->space->is_closing());
   mysql_mutex_assert_owner(&fil_system.mutex);
   static_assert(((UNIV_ZIP_SIZE_MIN >> 1) << 3) == 4096, "compatibility");
-#if defined _WIN32 || defined HAVE_FCNTL_DIRECT
+#if defined _WIN32 || defined O_DIRECT
   ulint type;
   switch (FSP_FLAGS_GET_ZIP_SSIZE(node->space->flags)) {
   case 1:
@@ -361,8 +361,7 @@ static bool fil_node_open_file_low(fil_node_t *node)
     bool success;
     node->handle= os_file_create(innodb_data_file_key, node->name,
                                  node->is_raw_disk
-                                 ? OS_FILE_OPEN_RAW | OS_FILE_ON_ERROR_NO_EXIT
-                                 : OS_FILE_OPEN | OS_FILE_ON_ERROR_NO_EXIT,
+                                 ? OS_FILE_OPEN_RAW : OS_FILE_OPEN,
                                  OS_FILE_AIO, type,
                                  srv_read_only_mode, &success);
 
@@ -945,9 +944,7 @@ bool fil_space_free(uint32_t id, bool x_latched)
 
 			log_sys.latch.wr_unlock();
 		} else {
-#ifndef SUX_LOCK_GENERIC
-			ut_ad(log_sys.latch.is_write_locked());
-#endif
+			ut_ad(log_sys.latch_have_wr());
 			if (space->max_lsn) {
 				ut_d(space->max_lsn = 0);
 				fil_system.named_spaces.remove(*space);
@@ -1249,9 +1246,6 @@ void fil_system_t::create(ulint hash_size)
 	ut_ad(!is_initialised());
 	ut_ad(!(srv_page_size % FSP_EXTENT_SIZE));
 	ut_ad(srv_page_size);
-	ut_ad(!spaces.array);
-
-	m_initialised = true;
 
 	compile_time_assert(!(UNIV_PAGE_SIZE_MAX % FSP_EXTENT_SIZE_MAX));
 	compile_time_assert(!(UNIV_PAGE_SIZE_MIN % FSP_EXTENT_SIZE_MIN));
@@ -1262,6 +1256,8 @@ void fil_system_t::create(ulint hash_size)
 
 	spaces.create(hash_size);
 
+	need_unflushed_spaces = !write_through && buf_dblwr.need_fsync();
+
 	fil_space_crypt_init();
 #ifdef __linux__
 	ssd.clear();
@@ -1335,13 +1331,12 @@ void fil_system_t::close()
 
   if (is_initialised())
   {
-    m_initialised= false;
     spaces.free();
     mysql_mutex_destroy(&mutex);
     fil_space_crypt_cleanup();
   }
 
-  ut_ad(!spaces.array);
+  ut_ad(!is_initialised());
 
 #ifdef __linux__
   ssd.clear();
@@ -1410,10 +1405,12 @@ ATTRIBUTE_COLD void fil_space_t::reopen_all()
 
       ulint type= OS_DATA_FILE;
 
+#if defined _WIN32 || defined O_DIRECT
       switch (FSP_FLAGS_GET_ZIP_SSIZE(space.flags)) {
       case 1: case 2:
         type= OS_DATA_FILE_NO_O_DIRECT;
       }
+#endif
 
       for (ulint count= 10000; count--;)
       {
@@ -1480,6 +1477,7 @@ void fil_system_t::set_write_through(bool write_through)
   {
     this->write_through= write_through;
     fil_space_t::reopen_all();
+    need_unflushed_spaces = !write_through && buf_dblwr.need_fsync();
   }
 
   mysql_mutex_unlock(&mutex);
@@ -1819,30 +1817,27 @@ pfs_os_file_t fil_delete_tablespace(uint32_t id)
 /*******************************************************************//**
 Allocates and builds a file name from a path, a table or tablespace name
 and a suffix. The string must be freed by caller with ut_free().
-@param[in] path NULL or the directory path or the full path and filename.
+@param[in] path nullptr or the directory path or the full path and filename
 @param[in] name {} if path is full, or Table/Tablespace name
-@param[in] ext the file extension to use
-@param[in] trim_name true if the last name on the path should be trimmed.
+@param[in] extension the file extension to use
+@param[in] trim_name true if the last name on the path should be trimmed
 @return own: file name */
-char* fil_make_filepath(const char *path, const fil_space_t::name_type &name,
-                        ib_extention ext, bool trim_name)
+char* fil_make_filepath_low(const char *path,
+                            const fil_space_t::name_type &name,
+                            ib_extention extension, bool trim_name)
 {
 	/* The path may contain the basename of the file, if so we do not
 	need the name.  If the path is NULL, we can use the default path,
 	but there needs to be a name. */
 	ut_ad(path || name.data());
 
-	/* If we are going to strip a name off the path, there better be a
-	path and a new name to put back on. */
-	ut_ad(!trim_name || (path && name.data()));
-
 	if (path == NULL) {
 		path = fil_path_to_mysql_datadir;
 	}
 
 	ulint	len		= 0;	/* current length */
 	ulint	path_len	= strlen(path);
-	const char* suffix	= dot_ext[ext];
+	const char* suffix	= dot_ext[extension];
 	ulint	suffix_len	= strlen(suffix);
 	ulint	full_len	= path_len + 1 + name.size() + suffix_len + 1;
 
@@ -1925,8 +1920,16 @@ char* fil_make_filepath(const char *path, const fil_space_t::name_type &name,
 char *fil_make_filepath(const char* path, const table_name_t name,
                         ib_extention suffix, bool strip_name)
 {
-  return fil_make_filepath(path, {name.m_name, strlen(name.m_name)},
-                           suffix, strip_name);
+  return fil_make_filepath_low(path, {name.m_name, strlen(name.m_name)},
+                               suffix, strip_name);
+}
+
+/** Wrapper function over fil_make_filepath_low() to build directory name.
+@param path the directory path or the full path and filename
+@return own: directory name */
+static inline char *fil_make_dirpath(const char *path)
+{
+  return fil_make_filepath_low(path, fil_space_t::name_type{}, NO_EXT, true);
 }
 
 dberr_t fil_space_t::rename(const char *path, bool log, bool replace)
@@ -1967,14 +1970,32 @@ dberr_t fil_space_t::rename(const char *path, bool log, bool replace)
     return DB_TABLESPACE_NOT_FOUND;
   }
 
-  exists= false;
-  if (replace);
-  else if (!os_file_status(path, &exists, &ftype) || exists)
+  if (!replace)
   {
-    sql_print_error("InnoDB: Cannot rename '%s' to '%s'"
-                    " because the target file exists.",
-                    old_path, path);
-    return DB_TABLESPACE_EXISTS;
+    char *schema_path= fil_make_dirpath(path);
+    if (!schema_path)
+      return DB_ERROR;
+
+    exists= false;
+    bool schema_fail= os_file_status(schema_path, &exists, &ftype) && !exists;
+    ut_free(schema_path);
+
+    if (schema_fail)
+    {
+      sql_print_error("InnoDB: Cannot rename '%s' to '%s'"
+                      " because the target schema directory doesn't exist.",
+                      old_path, path);
+      return DB_ERROR;
+    }
+
+    exists= false;
+    if (!os_file_status(path, &exists, &ftype) || exists)
+    {
+      sql_print_error("InnoDB: Cannot rename '%s' to '%s'"
+                      " because the target file exists.",
+                      old_path, path);
+      return DB_TABLESPACE_EXISTS;
+    }
   }
 
   mtr_t mtr;
@@ -2034,7 +2055,7 @@ fil_ibd_create(
 
 	static_assert(((UNIV_ZIP_SIZE_MIN >> 1) << 3) == 4096,
 		      "compatibility");
-#if defined _WIN32 || defined HAVE_FCNTL_DIRECT
+#if defined _WIN32 || defined O_DIRECT
 	ulint type;
 	switch (FSP_FLAGS_GET_ZIP_SSIZE(flags)) {
 	case 1:
@@ -2050,7 +2071,7 @@ fil_ibd_create(
 
 	file = os_file_create(
 		innodb_data_file_key, path,
-		OS_FILE_CREATE | OS_FILE_ON_ERROR_NO_EXIT,
+		OS_FILE_CREATE,
 		OS_FILE_AIO, type, srv_read_only_mode, &success);
 
 	if (!success) {
@@ -2826,19 +2847,18 @@ static void fil_invalid_page_access_msg(const char *name,
 }
 
 /** Update the data structures on write completion */
-inline void fil_node_t::complete_write()
+void fil_space_t::complete_write()
 {
   mysql_mutex_assert_not_owner(&fil_system.mutex);
 
-  if (space->purpose != FIL_TYPE_TEMPORARY &&
-      (!fil_system.is_write_through() && !my_disable_sync) &&
-      space->set_needs_flush())
+  if (purpose != FIL_TYPE_TEMPORARY &&
+      fil_system.use_unflushed_spaces() && set_needs_flush())
   {
     mysql_mutex_lock(&fil_system.mutex);
-    if (!space->is_in_unflushed_spaces)
+    if (!is_in_unflushed_spaces)
     {
-      space->is_in_unflushed_spaces= true;
-      fil_system.unflushed_spaces.push_front(*space);
+      is_in_unflushed_spaces= true;
+      fil_system.unflushed_spaces.push_front(*this);
     }
     mysql_mutex_unlock(&fil_system.mutex);
   }
@@ -2938,7 +2958,7 @@ io_error:
 	if (!type.is_async()) {
 		if (type.is_write()) {
 release_sync_write:
-			node->complete_write();
+			complete_write();
 release:
 			release();
 			goto func_exit;
@@ -2958,21 +2978,28 @@ void IORequest::write_complete(int io_error) const
 {
   ut_ad(fil_validate_skip());
   ut_ad(node);
+  fil_space_t *space= node->space;
   ut_ad(is_write());
-  node->complete_write();
 
   if (!bpage)
   {
     ut_ad(!srv_read_only_mode);
     if (type == IORequest::DBLWR_BATCH)
+    {
       buf_dblwr.flush_buffered_writes_completed(*this);
+      /* Above, we already invoked os_file_flush() on the
+      doublewrite buffer if needed. */
+      goto func_exit;
+    }
     else
       ut_ad(type == IORequest::WRITE_ASYNC);
   }
   else
     buf_page_write_complete(*this, io_error);
 
-  node->space->release();
+  space->complete_write();
+ func_exit:
+  space->release();
 }
 
 void IORequest::read_complete(int io_error) const
@@ -3153,9 +3180,7 @@ void
 fil_names_dirty(
 	fil_space_t*	space)
 {
-#ifndef SUX_LOCK_GENERIC
-	ut_ad(log_sys.latch.is_write_locked());
-#endif
+	ut_ad(log_sys.latch_have_wr());
 	ut_ad(recv_recovery_is_on());
 	ut_ad(log_sys.get_lsn() != 0);
 	ut_ad(space->max_lsn == 0);
@@ -3169,9 +3194,7 @@ fil_names_dirty(
 tablespace was modified for the first time since fil_names_clear(). */
 ATTRIBUTE_NOINLINE ATTRIBUTE_COLD void mtr_t::name_write()
 {
-#ifndef SUX_LOCK_GENERIC
-  ut_ad(log_sys.latch.is_write_locked());
-#endif
+  ut_ad(log_sys.latch_have_wr());
   ut_d(fil_space_validate_for_mtr_commit(m_user_space));
   ut_ad(!m_user_space->max_lsn);
   m_user_space->max_lsn= log_sys.get_lsn();
@@ -3195,9 +3218,7 @@ ATTRIBUTE_COLD lsn_t fil_names_clear(lsn_t lsn)
 {
 	mtr_t	mtr;
 
-#ifndef SUX_LOCK_GENERIC
-	ut_ad(log_sys.latch.is_write_locked());
-#endif
+	ut_ad(log_sys.latch_have_wr());
 	ut_ad(lsn);
 	ut_ad(log_sys.is_latest());
 
