@@ -19,65 +19,32 @@
 #define SPD_INIT_ALLOC_ROOT(A, B, C, D)                                       \
   init_alloc_root(PSI_INSTRUMENT_ME, A, B, C, D)
 
+/** Maximum possible number of `SPIDER_DBTON`s available to use */
 #define SPIDER_DBTON_SIZE 15
 
 #ifndef SIZEOF_STORED_DOUBLE
 #define SIZEOF_STORED_DOUBLE 8
 #endif
 
+/**
+  Possible wrapper values, e.g. for `SPIDER_DBTON::wrapper` and
+  `SPIDER_SHARE::tgt_wrappers`.
+
+  fixme: change this to enum
+*/
 #define SPIDER_DB_WRAPPER_MYSQL "mysql"
 #define SPIDER_DB_WRAPPER_MARIADB "mariadb"
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100204
 #define PLUGIN_VAR_CAN_MEMALLOC
-/*
-#define ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
-#define HASH_UPDATE_WITH_HASH_VALUE
-*/
-#else
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-#define HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-#endif
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100002
-#define SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
 #define SPIDER_HAS_APPEND_FOR_SINGLE_QUOTE
 #define SPIDER_HAS_SHOW_SIMPLE_FUNC
 #define SPIDER_HAS_JT_HASH_INDEX_MERGE
-#define SPIDER_HAS_EXPR_CACHE_ITEM
-#else
-#define SPIDER_NEED_CHECK_CONDITION_AT_CHECKING_DIRECT_ORDER_LIMIT
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100007
-#define SPIDER_ITEM_HAS_CMP_TYPE
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100004
 #define SPIDER_HAS_TIME_STATUS
 #define SPIDER_HAS_DECIMAL_OPERATION_RESULTS_VALUE_TYPE
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100014
-#define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100100
-#define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY_AND_THDPTR
-#endif
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100108
-#define SPIDER_HAS_GROUP_BY_HANDLER
-#endif
-
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100200
 #define SPIDER_ORDER_HAS_ENUM_ORDER
-#endif
-
-#if defined(MARIADB_BASE_VERSION)
 #define SPIDER_ITEM_GEOFUNC_NAME_HAS_MBR
 #define SPIDER_HANDLER_AUTO_REPAIR_HAS_ERROR
-#endif
 
 class spider_db_conn;
 typedef spider_db_conn SPIDER_DB_CONN;
@@ -119,7 +86,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_HS_LTEQUAL_STR "<="
 #define SPIDER_SQL_HS_LTEQUAL_LEN (sizeof(SPIDER_SQL_HS_LTEQUAL_STR) - 1)
 
-#ifdef ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
 #define SPIDER_SQL_CASE_STR "case "
 #define SPIDER_SQL_CASE_LEN (sizeof(SPIDER_SQL_CASE_STR) - 1)
 #define SPIDER_SQL_WHEN_STR " when "
@@ -130,7 +96,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_ELSE_LEN (sizeof(SPIDER_SQL_ELSE_STR) - 1)
 #define SPIDER_SQL_END_STR " end"
 #define SPIDER_SQL_END_LEN (sizeof(SPIDER_SQL_END_STR) - 1)
-#endif
 
 #define SPIDER_SQL_USING_STR " using "
 #define SPIDER_SQL_USING_LEN (sizeof(SPIDER_SQL_USING_STR) - 1)
@@ -208,11 +173,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_LOP_CHK_PRM_PRF_STR "spider_lc_"
 #define SPIDER_SQL_LOP_CHK_PRM_PRF_LEN (sizeof(SPIDER_SQL_LOP_CHK_PRM_PRF_STR) - 1)
 
-#define SPIDER_CONN_KIND_MYSQL (1 << 0)
-
-#define SPIDER_SQL_KIND_SQL (1 << 0)
-#define SPIDER_SQL_KIND_HANDLER (1 << 1)
-
 #define SPIDER_SQL_TYPE_SELECT_SQL (1 << 0)
 #define SPIDER_SQL_TYPE_INSERT_SQL (1 << 1)
 #define SPIDER_SQL_TYPE_UPDATE_SQL (1 << 2)
@@ -221,12 +181,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_TYPE_TMP_SQL (1 << 5)
 #define SPIDER_SQL_TYPE_DROP_TMP_TABLE_SQL (1 << 6)
 #define SPIDER_SQL_TYPE_OTHER_SQL (1 << 7)
-#define SPIDER_SQL_TYPE_HANDLER (1 << 8)
-#define SPIDER_SQL_TYPE_SELECT_HS (1 << 9)
-#define SPIDER_SQL_TYPE_INSERT_HS (1 << 10)
-#define SPIDER_SQL_TYPE_UPDATE_HS (1 << 11)
-#define SPIDER_SQL_TYPE_DELETE_HS (1 << 12)
-#define SPIDER_SQL_TYPE_OTHER_HS (1 << 13)
 
 enum spider_bulk_upd_start {
   SPD_BU_NOT_START,
@@ -427,12 +381,6 @@ public:
     IO_CACHE *file,
     uint32 arg_length
   );
-  bool append_with_prefill(
-    const char *s,
-    uint32 arg_length,
-    uint32 full_length,
-    char fill_char
-  );
   int strstr(
     const String &search,
     uint32 offset = 0
@@ -592,34 +540,26 @@ typedef struct spider_conn_holder
   spider_conn_holder *next;
 } SPIDER_CONN_HOLDER;
 
+/* Record information of a local (spider) table, for use of the spider
+group by handler. */
 typedef struct spider_table_holder
 {
   TABLE *table;
   ha_spider *spider;
+  /* alias of the table, in the form of tk, where k is the index of
+  the table from `query->from' indexed by next_local. */
   spider_string *alias;
 } SPIDER_TABLE_HOLDER;
 
-typedef struct spider_field_holder
-{
-  Field *field;
-  ha_spider *spider;
-  spider_string *alias;
-  spider_field_holder *next;
-} SPIDER_FIELD_HOLDER;
-
-typedef struct spider_field_chain
-{
-  spider_field_holder *field_holder;
-  spider_field_chain *next;
-} SPIDER_FIELD_CHAIN;
-
+/* For use of the spider group by handler. */
 class spider_fields
 {
   uint dbton_count;
   uint current_dbton_num;
   uint dbton_ids[SPIDER_DBTON_SIZE];
+  /* Number of tables in `query->from'. */
   uint table_count;
-  uint current_table_num;
+  /* All tables in `query->from', in the same order by next_local. */
   SPIDER_TABLE_HOLDER *table_holder;
   SPIDER_LINK_IDX_CHAIN *first_link_idx_chain;
   SPIDER_LINK_IDX_CHAIN *last_link_idx_chain;
@@ -628,13 +568,6 @@ class spider_fields
   SPIDER_CONN_HOLDER *first_conn_holder;
   SPIDER_CONN_HOLDER *last_conn_holder;
   SPIDER_CONN_HOLDER *current_conn_holder;
-  SPIDER_FIELD_HOLDER *first_field_holder;
-  SPIDER_FIELD_HOLDER *last_field_holder;
-  SPIDER_FIELD_HOLDER *current_field_holder;
-  SPIDER_FIELD_CHAIN *first_field_chain;
-  SPIDER_FIELD_CHAIN *last_field_chain;
-  SPIDER_FIELD_CHAIN *current_field_chain;
-  Field **first_field_ptr;
   Field **current_field_ptr;
 public:
   spider_fields();
@@ -677,8 +610,6 @@ public:
     long access_balance
   );
   SPIDER_CONN_HOLDER *create_conn_holder();
-  void set_pos_to_first_conn_holder();
-  SPIDER_CONN_HOLDER *get_next_conn_holder();
   bool has_conn_holder();
   void clear_conn_holder_from_conn();
   bool check_conn_same_conn(
@@ -692,24 +623,14 @@ public:
   void free_conn_holder(
     SPIDER_CONN_HOLDER *conn_holder_arg
   );
-  SPIDER_TABLE_HOLDER *add_table(
-    ha_spider *spider_arg
-  );
-  bool all_query_fields_are_query_table_members();
-  int create_table_holder(
+  SPIDER_TABLE_HOLDER *find_table(Field *field);
+  void set_table_holder(
+    SPIDER_TABLE_HOLDER *table_holder_arg,
     uint table_count_arg
   );
-  void set_pos_to_first_table_holder();
-  SPIDER_TABLE_HOLDER *get_next_table_holder();
+  SPIDER_TABLE_HOLDER *get_first_table_holder();
   SPIDER_TABLE_HOLDER *get_table_holder(TABLE *table);
   uint get_table_count();
-  int add_field(Field *field_arg);
-  SPIDER_FIELD_HOLDER *create_field_holder();
-  void set_pos_to_first_field_holder();
-  SPIDER_FIELD_HOLDER *get_next_field_holder();
-  SPIDER_FIELD_CHAIN *create_field_chain();
-  void set_pos_to_first_field_chain();
-  SPIDER_FIELD_CHAIN *get_next_field_chain();
   void set_field_ptr(Field **field_arg);
   Field **get_next_field_ptr();
   int ping_table_mon_from_table(
@@ -729,6 +650,8 @@ struct st_spider_db_request_key
 class spider_db_util
 {
 public:
+  /** Same as the `SPIDER_DBTON::dbton_id` of the `SPIDER_DBTON`
+  containing this `spider_db_util` */
   uint dbton_id;
   spider_db_util() = default;
   virtual ~spider_db_util() = default;
@@ -827,7 +750,6 @@ public:
     bool use_fields,
     spider_fields *fields
   ) = 0;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   virtual int open_item_sum_func(
     Item_sum *item_sum,
     ha_spider *spider,
@@ -837,12 +759,10 @@ public:
     bool use_fields,
     spider_fields *fields
   ) = 0;
-#endif
   virtual int append_escaped_util(
     spider_string *to,
     String *from
   ) = 0;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int append_from_and_tables(
     ha_spider *spider,
     spider_fields *fields,
@@ -850,18 +770,12 @@ public:
     TABLE_LIST *table_list,
     uint table_count
   ) = 0;
-  virtual int reappend_tables(
-    spider_fields *fields,
-    SPIDER_LINK_IDX_CHAIN *link_idx_chain,
-    spider_string *str
-  ) = 0;
   virtual int append_where(
     spider_string *str
   ) = 0;
   virtual int append_having(
     spider_string *str
   ) = 0;
-#endif
   virtual bool tables_on_different_db_are_joinable();
   virtual bool socket_has_default_value();
   virtual bool database_has_default_value();
@@ -944,11 +858,9 @@ public:
     int mode,
     ha_rows &records
   ) = 0;
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   virtual int fetch_table_checksum(
     ha_spider *spider
   );
-#endif
   virtual int fetch_table_cardinality(
     int mode,
     TABLE *table,
@@ -965,7 +877,6 @@ public:
     longlong pos
   ) = 0;
   virtual int get_errno() = 0;
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
   virtual int fetch_columns_for_discover_table_structure(
     spider_string *str,
     CHARSET_INFO *access_charset
@@ -979,7 +890,6 @@ public:
     SPIDER_SHARE *spider_share,
     CHARSET_INFO *access_charset
   ) = 0;
-#endif
   virtual uint limit_mode();
 };
 
@@ -1114,10 +1024,6 @@ public:
     Time_zone *time_zone,
     int *need_mon
   ) = 0;
-  virtual bool set_loop_check_in_bulk_sql();
-  virtual int set_loop_check(
-    int *need_mon
-  );
   virtual int fin_loop_check();
   virtual int show_master_status(
     SPIDER_TRX *trx,
@@ -1144,8 +1050,6 @@ public:
   ) = 0;
   virtual uint get_lock_table_hash_count() = 0;
   virtual void reset_lock_table_hash() = 0;
-  virtual uint get_opened_handler_count() = 0;
-  virtual void reset_opened_handler() = 0;
   virtual void set_dup_key_idx(
     ha_spider *spider,
     int link_idx
@@ -1186,16 +1090,12 @@ public:
     uint alias_length
   ) = 0;
   virtual bool need_change_db_table_name() = 0;
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
   virtual int discover_table_structure(
     SPIDER_TRX *trx,
     SPIDER_SHARE *spider_share,
     spider_string *str
   ) = 0;
-#endif
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   virtual bool checksum_support();
-#endif
 };
 
 class spider_db_handler
@@ -1210,9 +1110,7 @@ public:
   ha_spider *spider;
   spider_db_share *db_share;
   int first_link_idx;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   SPIDER_LINK_IDX_CHAIN *link_idx_chain;
-#endif
   bool strict_group_by;
   bool no_where_cond;
   spider_db_handler(ha_spider *spider, spider_db_share *db_share) :
@@ -1264,7 +1162,6 @@ public:
   virtual int append_update_part() = 0;
   virtual int append_delete_part() = 0;
   virtual int append_update_set_part() = 0;
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   virtual int append_direct_update_set_part() = 0;
   virtual int append_dup_update_pushdown_part(
     const char *alias,
@@ -1275,7 +1172,6 @@ public:
     uint alias_length
   ) = 0;
   virtual int check_update_columns_part() = 0;
-#endif
   virtual int append_select_part(
     ulong sql_type
   ) = 0;
@@ -1357,26 +1253,22 @@ public:
     const char *alias,
     uint alias_length
   ) = 0;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   virtual int append_sum_select_part(
     ulong sql_type,
     const char *alias,
     uint alias_length
   ) = 0;
-#endif
   virtual void set_order_pos(
     ulong sql_type
   ) = 0;
   virtual void set_order_to_pos(
     ulong sql_type
   ) = 0;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   virtual int append_group_by_part(
     const char *alias,
     uint alias_length,
     ulong sql_type
   ) = 0;
-#endif
   virtual int append_key_order_for_merge_with_alias_part(
     const char *alias,
     uint alias_length,
@@ -1423,16 +1315,6 @@ public:
     ulong sql_type,
     uint multi_range_cnt
   ) = 0;
-  virtual int append_open_handler_part(
-    ulong sql_type,
-    uint handler_id,
-    SPIDER_CONN *conn,
-    int link_idx
-  ) = 0;
-  virtual int append_close_handler_part(
-    ulong sql_type,
-    int link_idx
-  ) = 0;
   virtual int append_insert_terminator_part(
     ulong sql_type
   ) = 0;
@@ -1501,16 +1383,11 @@ public:
   virtual int reset_sql(
     ulong sql_type
   ) = 0;
-  virtual bool need_lock_before_set_sql_for_exec(
-    ulong sql_type
-  ) = 0;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int set_sql_for_exec(
     ulong sql_type,
     int link_idx,
     SPIDER_LINK_IDX_CHAIN *link_idx_chain
   ) = 0;
-#endif
   virtual int set_sql_for_exec(
     ulong sql_type,
     int link_idx
@@ -1544,11 +1421,9 @@ public:
   virtual int show_records(
     int link_idx
   ) = 0;
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   virtual int checksum_table(
     int link_idx
   );
-#endif
   virtual int show_last_insert_id(
     int link_idx,
     ulonglong &last_insert_id
@@ -1599,20 +1474,9 @@ public:
     SPIDER_CONN *conn,
     int link_idx
   ) = 0;
-  virtual int insert_opened_handler(
-    SPIDER_CONN *conn,
-    int link_idx
-  ) = 0;
-  virtual int delete_opened_handler(
-    SPIDER_CONN *conn,
-    int link_idx
-  ) = 0;
   virtual int sync_from_clone_source(
     spider_db_handler *dbton_hdl
   ) = 0;
-  virtual bool support_use_handler(
-    int use_handler
-  ) = 0;
   virtual bool minimum_select_bit_is_set(
     uint field_index
   ) = 0;
@@ -1626,15 +1490,10 @@ public:
     int link_idx,
     ulong sql_type
   ) = 0;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int append_from_and_tables_part(
     spider_fields *fields,
     ulong sql_type
   ) = 0;
-  virtual int reappend_tables_part(
-    spider_fields *fields,
-    ulong sql_type
-  ) = 0;
   virtual int append_where_part(
     ulong sql_type
   ) = 0;
@@ -1673,8 +1532,6 @@ public:
     spider_fields *fields,
     ulong sql_type
   ) = 0;
-#endif
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   virtual bool check_direct_update(
     st_select_lex *select_lex,
     longlong select_limit,
@@ -1685,7 +1542,6 @@ public:
     longlong select_limit,
     longlong offset_limit
   );
-#endif
 };
 
 class spider_db_copy_table
@@ -1778,9 +1634,18 @@ static const LEX_CSTRING maturity_name[] =
   { STRING_WITH_LEN("Stable") }
 };
 
+/*
+  Type of singletons based on the type of the remote database.
+
+  All such singletons are stored in the array `spider_dbton', see
+  `spider_db_init()'.
+*/
 typedef struct st_spider_dbton
 {
+  /** The index of this dbton in `spider_dbton` */
   uint dbton_id;
+  /** The wrapper of this dbton, same possible values as each element
+  of `SPIDER_SHARE::tgt_wrappers` */
   const char *wrapper;
   enum spider_db_access_type db_access_type;
   int (*init)();
@@ -1804,10 +1669,7 @@ typedef struct st_spider_position
   uint                   pos_mode;
   bool                   use_position;
   bool                   mrr_with_cnt;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   bool                   direct_aggregate;
-#endif
-  uint                   sql_kind;
   uchar                  *position_bitmap;
   st_spider_ft_info      *ft_first;
   st_spider_ft_info      *ft_current;
@@ -1825,13 +1687,9 @@ typedef struct st_spider_result
 {
   uint                 dbton_id;
   SPIDER_DB_RESULT     *result;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     st_spider_result   *prev;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     st_spider_result   *next;
   SPIDER_POSITION      *first_position; /* for quick mode */
   int                  pos_page_size; /* for quick mode */
@@ -1850,25 +1708,16 @@ typedef struct st_spider_result
 
 typedef struct st_spider_result_list
 {
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     SPIDER_RESULT        *first;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     SPIDER_RESULT        *last;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     SPIDER_RESULT         *current;
   KEY                     *key_info;
   int                     key_order;
   spider_string           *sqls;
   int                     ha_read_kind;
-  bool                    have_sql_kind_backup;
-  uint                    *sql_kind_backup;
-  uint                    sql_kinds_backup;
   bool                    use_union;
   bool                    use_both_key;
   const key_range         *end_key;
@@ -1907,12 +1756,10 @@ typedef struct st_spider_result_list
   /* the limit_offeset, without where condition */
   bool                    direct_limit_offset;
   bool                    direct_distinct;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   bool                    direct_aggregate;
   bool                    snap_mrr_with_cnt;
   bool                    snap_direct_aggregate;
   SPIDER_DB_ROW           *snap_row;
-#endif
   bool                    in_cmp_ref;
   bool                    set_split_read;
   bool                    insert_dup_update_pushdown;
@@ -1924,14 +1771,11 @@ typedef struct st_spider_result_list
   longlong                second_read;
   int                     set_split_read_count;
   int                     *casual_read;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   /* 0:nomal 1:store 2:store end */
   volatile
-#endif
     int                   quick_phase;
   bool                    keyread;
   TABLE                   *table;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile int            bgs_error;
   bool                    bgs_error_with_message;
   char                    bgs_error_msg[MYSQL_ERRMSG_SIZE];
@@ -1942,7 +1786,6 @@ typedef struct st_spider_result_list
   volatile longlong       bgs_second_read;
   volatile longlong       bgs_split_read;
   volatile
-#endif
     SPIDER_RESULT         *bgs_current;
   SPIDER_DB_ROW           *tmp_pos_row_first;
 } SPIDER_RESULT_LIST;
