@@ -105,9 +105,7 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_mon_list(
   SPIDER_TABLE_MON_LIST *table_mon_list;
   MEM_ROOT mem_root;
   ulonglong mon_table_cache_version;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   my_hash_value_type hash_value;
-#endif
   DBUG_ENTER("spider_get_ping_table_mon_list");
   /* Reset the cache if the version does not match the requirement */
   if (spider_mon_table_cache_version != spider_mon_table_cache_version_req)
@@ -125,31 +123,21 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_mon_list(
   /* Search for the table mon list in the hash, if one is not found or
   if it is found but has the wrong cache version, create and
   initialise a new one. */
-  mutex_hash = spider_udf_calc_hash(str->c_ptr(),
-    spider_param_udf_table_mon_mutex_count());
+  mutex_hash=
+      spider_udf_calc_hash(str->c_ptr(), spider_udf_table_mon_mutex_count);
   DBUG_PRINT("info",("spider hash key=%s", str->c_ptr()));
   DBUG_PRINT("info",("spider hash key length=%u", str->length()));
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   hash_value = my_calc_hash(
     &spider_udf_table_mon_list_hash[mutex_hash],
     (uchar*) str->c_ptr(), str->length());
-#endif
   pthread_mutex_lock(&spider_udf_table_mon_mutexes[mutex_hash]);
   mon_table_cache_version = (ulonglong) spider_mon_table_cache_version;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   if (!(table_mon_list = (SPIDER_TABLE_MON_LIST *)
     my_hash_search_using_hash_value(
       &spider_udf_table_mon_list_hash[mutex_hash], hash_value,
       (uchar*) str->c_ptr(), str->length())) ||
       table_mon_list->mon_table_cache_version != mon_table_cache_version
   )
-#else
-  if (!(table_mon_list = (SPIDER_TABLE_MON_LIST *) my_hash_search(
-    &spider_udf_table_mon_list_hash[mutex_hash],
-    (uchar*) str->c_ptr(), str->length())) ||
-    table_mon_list->mon_table_cache_version != mon_table_cache_version
-  )
-#endif
   {
     /* If table_mon_list is found but the cache version does not
     match, remove it from the hash and free it. */
@@ -171,17 +159,9 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_mon_list(
     table_mon_list->mon_table_cache_version = mon_table_cache_version;
     uint old_elements =
       spider_udf_table_mon_list_hash[mutex_hash].array.max_element;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
     table_mon_list->key_hash_value = hash_value;
-#endif
-#ifdef HASH_UPDATE_WITH_HASH_VALUE
-    if (my_hash_insert_with_hash_value(
-      &spider_udf_table_mon_list_hash[mutex_hash],
-      hash_value, (uchar*) table_mon_list))
-#else
     if (my_hash_insert(&spider_udf_table_mon_list_hash[mutex_hash],
       (uchar*) table_mon_list))
-#endif
     {
       spider_ping_table_free_mon_list(table_mon_list);
       *error_num = HA_ERR_OUT_OF_MEM;
@@ -230,13 +210,8 @@ void spider_release_ping_table_mon_list_loop(
   SPIDER_TABLE_MON_LIST *table_mon_list
 ) {
   DBUG_ENTER("spider_release_ping_table_mon_list_loop");
-#ifdef HASH_UPDATE_WITH_HASH_VALUE
-  my_hash_delete_with_hash_value(&spider_udf_table_mon_list_hash[mutex_hash],
-    table_mon_list->key_hash_value, (uchar*) table_mon_list);
-#else
   my_hash_delete(&spider_udf_table_mon_list_hash[mutex_hash],
     (uchar*) table_mon_list);
-#endif
   while (TRUE)
   {
     if (table_mon_list->use_count)
@@ -278,24 +253,16 @@ int spider_release_ping_table_mon_list(
   conv_name_str.q_append(conv_name, conv_name_length);
   conv_name_str.q_append(link_idx_str, link_idx_str_length);
 
-  mutex_hash = spider_udf_calc_hash(conv_name_str.c_ptr_safe(),
-    spider_param_udf_table_mon_mutex_count());
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
+  mutex_hash= spider_udf_calc_hash(conv_name_str.c_ptr_safe(),
+                                   spider_udf_table_mon_mutex_count);
   my_hash_value_type hash_value = my_calc_hash(
     &spider_udf_table_mon_list_hash[mutex_hash],
     (uchar*) conv_name_str.c_ptr(), conv_name_str.length());
-#endif
   pthread_mutex_lock(&spider_udf_table_mon_mutexes[mutex_hash]);
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   if ((table_mon_list = (SPIDER_TABLE_MON_LIST *)
     my_hash_search_using_hash_value(
       &spider_udf_table_mon_list_hash[mutex_hash], hash_value,
       (uchar*) conv_name_str.c_ptr(), conv_name_str.length())))
-#else
-  if ((table_mon_list = (SPIDER_TABLE_MON_LIST *) my_hash_search(
-    &spider_udf_table_mon_list_hash[mutex_hash],
-    (uchar*) conv_name_str.c_ptr(), conv_name_str.length())))
-#endif
     spider_release_ping_table_mon_list_loop(mutex_hash, table_mon_list);
   pthread_mutex_unlock(&spider_udf_table_mon_mutexes[mutex_hash]);
   my_afree(buf);
@@ -337,7 +304,7 @@ int spider_get_ping_table_mon(
     !(table_link_mon = spider_open_sys_table(
       thd, SPIDER_SYS_LINK_MON_TABLE_NAME_STR,
       SPIDER_SYS_LINK_MON_TABLE_NAME_LEN, FALSE, &open_tables_backup,
-      need_lock, &error_num))
+      &error_num))
   ) {
     my_error(error_num, MYF(0));
     goto error;
@@ -440,10 +407,8 @@ create_table_mon:
     if (
       (error_num = spider_set_connect_info_default(
         tmp_share,
-#ifdef WITH_PARTITION_STORAGE_ENGINE
         NULL,
         NULL,
-#endif
         NULL
       )) ||
       (error_num = spider_set_connect_info_default_dbtable(
@@ -463,8 +428,7 @@ create_table_mon:
     error_num = spider_sys_index_next_same(table_link_mon, table_key);
   } while (error_num == 0);
   spider_sys_index_end(table_link_mon);
-  spider_close_sys_table(thd, table_link_mon,
-    &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   table_link_mon = NULL;
   table_mon_list->list_size = list_size;
 
@@ -480,8 +444,7 @@ create_table_mon:
 
 error:
   if (table_link_mon)
-    spider_close_sys_table(thd, table_link_mon,
-      &open_tables_backup, need_lock);
+    spider_sys_close_table(thd, &open_tables_backup);
   table_mon = table_mon_list->first;
   table_mon_list->first = NULL;
   table_mon_list->current = NULL;
@@ -558,7 +521,7 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
   if (
     !(table_tables = spider_open_sys_table(
       thd, SPIDER_SYS_TABLES_TABLE_NAME_STR,
-      SPIDER_SYS_TABLES_TABLE_NAME_LEN, FALSE, &open_tables_backup, need_lock,
+      SPIDER_SYS_TABLES_TABLE_NAME_LEN, FALSE, &open_tables_backup,
       error_num))
   ) {
     my_error(*error_num, MYF(0));
@@ -594,22 +557,19 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
     (*error_num = spider_get_sys_tables_connect_info(
       table_tables, tmp_share, &mem_root)) ||
     (*error_num = spider_get_sys_tables_link_status(
-      table_tables, tmp_share, 0, &mem_root))
+      table_tables, tmp_share->link_statuses, &mem_root))
   ) {
     table_tables->file->print_error(*error_num, MYF(0));
     goto error;
   }
-  spider_close_sys_table(thd, table_tables,
-    &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   table_tables = NULL;
 
   if (
     (*error_num = spider_set_connect_info_default(
       tmp_share,
-#ifdef WITH_PARTITION_STORAGE_ENGINE
       NULL,
       NULL,
-#endif
       NULL
     )) ||
     (*error_num = spider_set_connect_info_default_dbtable(
@@ -664,8 +624,7 @@ error_receptor_mutex_init:
 error_caller_mutex_init:
 error:
   if (table_tables)
-    spider_close_sys_table(thd, table_tables,
-      &open_tables_backup, need_lock);
+    spider_sys_close_table(thd, &open_tables_backup);
   free_root(&mem_root, MYF(0));
   if (table_mon_list)
   {
@@ -682,11 +641,9 @@ SPIDER_CONN *spider_get_ping_table_tgt_conn(
 ) {
   SPIDER_CONN *conn;
   DBUG_ENTER("spider_get_ping_table_tgt_conn");
-  if (
-    !(conn = spider_get_conn(
-      share, 0, share->conn_keys[0], trx, NULL, FALSE, FALSE,
-      SPIDER_CONN_KIND_MYSQL, error_num))
-  ) {
+  if (!(conn= spider_get_conn(share, 0, share->conn_keys[0], trx, NULL, FALSE,
+                              FALSE, error_num)))
+  {
     my_error(ER_CONNECT_TO_FOREIGN_DATA_SOURCE, MYF(0),
       share->server_names[0]);
     *error_num = ER_CONNECT_TO_FOREIGN_DATA_SOURCE;
@@ -881,11 +838,10 @@ int spider_get_ping_table_gtid_pos(
     goto error_sys_index_end;
   }
 #ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
-  spider_close_sys_table(thd, table_gtid_pos,
+  spider_sys_close_table(thd, table_gtid_pos,
     &open_tables_backup_gtid_pos, need_lock);
 #endif
-  spider_close_sys_table(thd, table_tables, &open_tables_backup_tables,
-    need_lock);
+  spider_sys_close_table(thd, &open_tables_backup_tables);
 
   DBUG_RETURN(0);
 
@@ -896,13 +852,12 @@ error_get_sys_tables_link_status:
 error_sys_index_end:
 error_get_sys_table_by_idx:
 #ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
-  spider_close_sys_table(thd, table_gtid_pos,
+  spider_sys_close_table(thd, table_gtid_pos,
     &open_tables_backup_gtid_pos,
     need_lock);
 error_open_table_gtid_pos:
 #endif
-  spider_close_sys_table(thd, table_tables, &open_tables_backup_tables,
-    need_lock);
+  spider_sys_close_table(thd, &open_tables_backup_tables);
 error_open_table_tables:
   DBUG_RETURN(error_num);
 }
@@ -928,7 +883,7 @@ int spider_init_ping_table_mon_cache(
     !(table_link_mon = spider_open_sys_table(
       thd, SPIDER_SYS_LINK_MON_TABLE_NAME_STR,
       SPIDER_SYS_LINK_MON_TABLE_NAME_LEN, FALSE, &open_tables_backup,
-      need_lock, &error_num))
+      &error_num))
   ) {
     my_error(error_num, MYF(0));
     goto error_open_sys_table;
@@ -1023,7 +978,7 @@ int spider_init_ping_table_mon_cache(
     spider_mon_table_cache_version = spider_mon_table_cache_version_req;
   }
   pthread_mutex_unlock(&spider_mon_table_cache_mutex);
-  spider_close_sys_table(thd, table_link_mon, &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
   DBUG_RETURN(0);
 
 error_push_dynamic:
@@ -1032,7 +987,7 @@ error_sys_index_next:
   spider_sys_index_end(table_link_mon);
 error_sys_index_first:
   pthread_mutex_unlock(&spider_mon_table_cache_mutex);
-  spider_close_sys_table(thd, table_link_mon, &open_tables_backup, need_lock);
+  spider_sys_close_table(thd, &open_tables_backup);
 error_open_sys_table:
   DBUG_RETURN(error_num);
 }
@@ -1329,9 +1284,9 @@ long long spider_ping_table_body(
               conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
             spider_sys_update_tables_link_status(trx->thd,
               conv_name.c_ptr(), conv_name_length, link_idx,
-              SPIDER_LINK_STATUS_NG, TRUE);
+              SPIDER_LINK_STATUS_NG);
             spider_sys_log_tables_link_failed(trx->thd,
-              conv_name.c_ptr(), conv_name_length, link_idx, TRUE);
+              conv_name.c_ptr(), conv_name_length, link_idx);
             status_changed_to_ng = TRUE;
           }
           pthread_mutex_unlock(&spider_udf_table_mon_mutexes[table_mon_list->mutex_hash]);
@@ -1396,9 +1351,9 @@ long long spider_ping_table_body(
                 conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
               spider_sys_update_tables_link_status(trx->thd,
                 conv_name.c_ptr(), conv_name_length, link_idx,
-                SPIDER_LINK_STATUS_NG, TRUE);
+                SPIDER_LINK_STATUS_NG);
               spider_sys_log_tables_link_failed(trx->thd,
-                conv_name.c_ptr(), conv_name_length, link_idx, TRUE);
+                conv_name.c_ptr(), conv_name_length, link_idx);
               status_changed_to_ng = TRUE;
             }
             pthread_mutex_unlock(&spider_udf_table_mon_mutexes[table_mon_list->mutex_hash]);
@@ -1453,9 +1408,9 @@ long long spider_ping_table_body(
                 conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
               spider_sys_update_tables_link_status(trx->thd,
                 conv_name.c_ptr(), conv_name_length, link_idx,
-                SPIDER_LINK_STATUS_NG, TRUE);
+                SPIDER_LINK_STATUS_NG);
               spider_sys_log_tables_link_failed(trx->thd,
-                conv_name.c_ptr(), conv_name_length, link_idx, TRUE);
+                conv_name.c_ptr(), conv_name_length, link_idx);
               status_changed_to_ng = TRUE;
             }
             pthread_mutex_unlock(&spider_udf_table_mon_mutexes[table_mon_list->mutex_hash]);
@@ -1803,9 +1758,9 @@ int spider_ping_table_mon_from_table(
                   link_idx));
                 share->link_statuses[link_idx] = SPIDER_LINK_STATUS_NG;
                 spider_sys_update_tables_link_status(thd, conv_name,
-                  conv_name_length, link_idx, SPIDER_LINK_STATUS_NG, need_lock);
+                  conv_name_length, link_idx, SPIDER_LINK_STATUS_NG);
                 spider_sys_log_tables_link_failed(thd, conv_name,
-                  conv_name_length, link_idx, need_lock);
+                  conv_name_length, link_idx);
               }
               pthread_mutex_unlock(&spider_udf_table_mon_mutexes[table_mon_list->mutex_hash]);
             }
